#
# ident "@(#)cb_prom.tcl 1.8     98/03/13 SMI"
#
# Copyright (c) 1996 by Sun Microsystems, Inc.
# All rights reserved.
#
# cb_prom
#
# Description:
#
#	cb_prom is a TCL/SSP command that allows the user to:
#
#	(1) Program the FLASH PROM on the Starfire Control Board with an
#	    ELF binary on the SSP.  After reset, the Control Board will
#	    then jump to the FLASH PROM and execute the boot sequence
#	    from that address space, instead of the default boot PROM.
#
#	(2) Erase the contents of the FLASH PROM.  This sets the contents
#	    of the entire chip to byte 0xFF.
#
#	(3) Dump the contents of the FLASH PROM to a file located on the
#	    SSP.
#
#	The program and erase sequences take several seconds to complete;
#	progress messages from the Control Board are written to
#	/var/opt/SUNWssp/adm/messages.  This tool makes use of the
#	/tftpboot directory on the SSP Workstation for transfering files,
#	so it should be writable by the ssp user.
#
# Usage:
#
#	cb_prom [-p file] [-e] [-d file]
#
# Options:
#
#	-p file	: Program the FLASH PROM with the contents of file.
#	-e	: Erase contents of FLASH PROM.
#	-d file : Dump the contents of the FLASH PROM to file.
#

#
# cb_prom command - main
#
proc cb_prom args {
	global env

	set cray_env "SUNW_HOSTNAME SSPETC SSPVAR"
	set valid_args "ep:d:"
	set usage "Usage: cb_prom \[-p file\] \[-e\] \[-d file\]"

	#
	# Check command line arguments
	#
	if { [catch {CLA_Parse Args $args $valid_args} errmsg] } {
		puts "Error: $errmsg"
		error $usage
	}

	#
	# Check SSP environment variables:
	#
	foreach var $cray_env {
		if { ![info exists env($var)] } {
			error "cb_prom: Environment variable '$var' must be set."
		}
	}

	#
	# Process command line arguments
	#

	# If no arguments supplied print usage message and exit
	if { ![array size Args] } {
		error $usage
	}

	if { [catch {cbs_connect $env(SUNW_HOSTNAME) "cb_prom"} handle] } {
		error "cb_prom: Could not connect to CBS"
	}

	if { [catch {cbs_set_timeout $handle 90} errmsg] } {
		error "cb_prom: Could not set CBS client timeout: $errmsg"
	}

	if { [info exists Args(p)] } {

		if { ![file readable $Args(p)] } {
			error "cb_prom: $Args(p): Error accessing file"
		}

		set tail [file tail $Args(p)]
	        if { ![string length $tail] } {
			error $usage
		}

		puts "cb_prom: Program PROM with file $Args(p)..."

		if { [catch {cbs_prog_flash $handle $tail} errmsg] } {
			error "$errmsg"
		}

		puts "cb_prom: Program complete."
	}

	if { [info exists Args(e)] } {
		puts "cb_prom: Erase PROM..."
		if { [catch {cbs_erase_flash $handle} errmsg] } {
			error "$errmsg"
		}
		puts "cb_prom: Erase complete."
	}

	if { [info exists Args(d)] } {
		set tail [file tail $Args(d)]

	        if { ![string length $tail] } {
			error $usage
		}

		puts "cb_prom: Dump PROM to $Args(d)..."

		if { [catch {cbs_dump_flash $handle $tail} errmsg] } {
			error "$errmsg"
		}

		puts "cb_prom: Dump complete."
	}

	cbs_disconnect $handle
}
